﻿using System;
using System.Diagnostics;
using System.Windows.Input;
using System.Xml.Linq;
using GarboChess;
using Microsoft.Research.Rin.DataModel;
using MicrosoftResearch.Rin.RinPlayerContracts;

namespace MicrosoftResearch.Rin.Samples
{
    /// <summary>
    /// Sample to show how to build an Experience Stream that works based on discrete keyframes. System base class DiscreteKeyframeESBase
    /// gives default implementation for all IExperienceStream members and takes care of parsing keyframes element and calling
    /// DisplayKeyframe as right times. Using this base class significantly reduces coding.
    /// </summary>
    public partial class ChessES : DiscreteKeyframeESBase, IAuthorableExperienceStream
    {
        public ChessES(ExperienceStreamData esData, IOrchestrator orchestrator)
            : base(esData, orchestrator)
        {
            InitializeComponent();
        }

        #region IAuthorableExperienceStream members
        /// <summary>
        /// Returns current state as a keyframe to be stored in RIN data file. Used in authoring scenario.
        /// </summary>
        public KeyframeData GetCapturedKeyframe(KeyframeData keyframeData)
        {
            XElement dataElement = null;
                dataElement = new XElement("kf",
                    new XAttribute("Fen", p.GetFen()));

            keyframeData.Data["default"] = dataElement;
            return keyframeData;
        }
        #endregion

        /// <summary>
        /// Override for base Load. Sets up chess board and controls.
        /// </summary>
        public override void Load(TimeSpan offset)
        {
            // Make sure to call base before custom code.
            base.Load(offset);

            p = new Position();
            p.ResetBoard();
            chessBoard.Position = p;

            chessBoard.UserMove += new SilverlightChess.ChessBoard.UserMoveHandler(cb_UserMove);
            chessBoard.MouseLeftButtonDown += new MouseButtonEventHandler(cb_MouseLeftButtonDown);
            this.MouseLeftButtonDown += cb_MouseLeftButtonDown;

            this.State = ExperienceStreamState.Ready;
        }

        /// <summary>
        /// Displays the currently passed keyframe in the chessboard.
        /// </summary>
        protected override void DisplayKeyframe(KeyframeData keyframeData)
        {
            base.DisplayKeyframe(keyframeData);

            XElement element = keyframeData.Data["default"];
            Debug.WriteLine("Keyframe at chess: " + element.ToString());

            string viewerState = element.Attribute("Fen").Value;
            p.InitializeFromFen(viewerState);
            chessBoard.UpdateForNewPosition();
        }

        /// <summary>
        /// Reset UI to default state.
        /// </summary>
        protected override void ResetUserInterface()
        {
            base.ResetUserInterface();

            p.ResetBoard();
        }

        /// <summary>
        /// Pauses narratives for user exploration when a mouse down occurs
        /// </summary>
        private void cb_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            this.orchestrator.PauseNarrativeForExplore();
        }

        /// <summary>
        /// Updates chessboard when user makes a move.
        /// </summary>
        private void cb_UserMove(ushort move)
        {
            p.MakeMove(move);
            chessBoard.UpdateForNewPosition();
        }

        private Position p;
    }
}
