﻿using System;
using System.ComponentModel.Composition;
using System.Diagnostics;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media.Animation;
using System.Windows.Threading;
using Microsoft.Research.Rin.DataModel;
using MicrosoftResearch.Rin.RinPlayerContracts;

namespace MicrosoftResearch.Rin.Samples
{
    /// <summary>
    /// Sample Experience Stream to demo RIN extensibility
    /// </summary>
    public partial class HelloWorldExperienceStream : UserControl, IExperienceStream
    {
        /// <summary>
        /// Constructor for the experience stream
        /// </summary>
        /// <param name="orchestrator">IOrchestrator is interface for host environment and has many optional functionalities that an ES may need. See documentation for complete list of members.</param>
        /// <param name="esData">data present in RIN data model for this ES</param>
        public HelloWorldExperienceStream(IOrchestrator orchestrator, ExperienceStreamData esData)
        {
            InitializeComponent();

            this.orchestrator = orchestrator;
            this.esData = esData;

            this.LayoutRoot.SizeChanged += new SizeChangedEventHandler(LayoutRoot_SizeChanged);
        }

        void LayoutRoot_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            this.storyboard = CreateMarqueeStoryboard(this.textBlockMain, this.LayoutRoot);
        }
        
        
        #region IExperienceStream Members

        /// <summary>
        /// ES to load the media data into memory in this call and move it to the requested offset.
        /// The experience stream should show the first frame pointed out by "offset" parameter or show default content after load
        /// but should not play content automatically.
        /// The ES should raise ESStateChanged event to notify listeners of this change.
        /// This method should return immediately and long running operations should not be
        /// done in this method. If the experience stream needs to connect to server, it should be done
        /// asynchronoulsy.
        /// 
        /// 1) If the content is not ready to play by end of this method, the experience stream 
        /// should change its state to "Buffering" and raise state changed event.
        /// 2) If the content is ready to play by end of this method, the experience stream
        /// should change its state to "Ready" and raise state changed event.
        /// 3) If the content getting operation failed during this method (or in an asynchronous call), the experience stream
        /// should change its state to "Error" and raise state changed event.
        /// Valid Before ES States: Closed
        /// Valid After ES States: Buffering, Ready, Error.
        /// 
        /// These methods are called by Orchestrator before this call: None
        /// </summary>
        /// <param name="offset">Time offset at which the ES should seek at beginning and try to be ready for Play()</param>
        public void Load(TimeSpan offset)
        {
            this.startTime = DateTime.Now;
            

            this.timer = new DispatcherTimer() { Interval = TimeSpan.FromSeconds(1) };
            this.timer.Tick += (object sender, EventArgs e) =>
            {
                WriteCurrentTimeStatus();
            };
            this.timer.Start();

            this.State = ExperienceStreamState.Ready;
        }

        /// <summary>
        /// ES to resume playing from paused state. If transitionTime = 0, then the resume should be immediate. 
        /// If transitionTime > 0, then the ES should take that much time to go to its last offset. In other words, the ES should
        /// not resume from its old offset rightaway.
        /// 
        /// If the ES does not have enough content to play within transitionTime, then the ES should 
        /// immediately move to Buffering state and raise ESStateChanged event.
        /// 
        /// Valid Before States: Buffering, Ready
        /// Valid After States: Buffering, Ready, Error
        /// 
        /// Methods called by Orchestrator before this call: Load, Pause (possibly)
        /// </summary>
        /// <param name="transitionTime"></param>
        public void Play(TimeSpan transitionTime)
        {
            Debug.Assert(this.State == ExperienceStreamState.Ready);
            if (this.isStoryboardStarted)
            {
                this.storyboard.Resume();
                this.textBlockMain.Text = transitionTime == TimeSpan.Zero ? "I am playing..." : "I am resumed...";
            }
            else
            {
                this.storyboard.Begin();
                this.textBlockMain.Text = "I am playing...";
                this.isStoryboardStarted = true;
            }
        }

        /// <summary>
        /// ES to pause the play in the time duration specified by transitionTime. If the transitionTime is 0, then the pause should
        /// be immediate. If it is more than zero, then the ES should try to smoothly transition to pause in that time.
        /// 
        /// Generally transitionTime=0 means player is going to buffering state and transitionTime > 0 means user is pausing to explore.
        /// 
        /// Valid Before States: Buffering, Ready
        /// Valid After States: Buffering, Ready, Error
        /// 
        /// These methods are called by Orchestrator before this call: Load, Play
        /// </summary>
        /// <param name="transitionTime"></param>
        public void Pause(TimeSpan transitionTime)
        {
            this.storyboard.Pause();
            this.textBlockMain.Text = "I am paused...";
        }

        /// <summary>
        /// ES to stop its play and optionally unload its contents from memory.
        /// 
        /// Valid Before States: Buffering, Ready
        /// Valid After States: Closed
        /// 
        /// These methods are called by Orchestrator before this call: Load, Play, Pause
        /// </summary>
        public void Unload()
        {
            this.storyboard.Stop();
            this.State = ExperienceStreamState.Closed;
        }

        /// <summary>
        /// ES to seek to the new location specified. A pause will be called before the seek. After the seek, the player
        /// will try to restore old player state as described below:
        /// 
        /// 1) If player was in "Playing" mode and a seek happened, then order of calls are: Pause(0), Seek(new offset), Resume(0)
        /// 2) If player was in "Paused for buffering" mode and a seek happened, then order of calls are: Pause(0), Seek(new offset).
        /// 3) If player was in "Paused for explore" mode and a seek happened, then order of calls are: Pause(0), Seek(new offset), Pause(transitionTime)
        /// 4) If player was in "Stopped" mode and a seek happened, then order of calls are: Load(0), Pause(0), Seek(new offset),Resume(0)
        ///
        /// Valid Before States: Buffering, Ready
        /// Valid After States: Buffering, Ready, Error
        /// 
        /// These methods are called by Orchestrator before this call: Load, Play, Pause
        /// </summary>
        /// <param name="offset">new offset to seek to</param>
        public void Seek(TimeSpan offset)
        {
            if (this.storyboard != null)
            {
                this.storyboard.Seek(offset);
            }
        }

        /// <summary>
        /// Current state of Experience Stream. whenever the state changes, the ES should raise ESStateChanged event.
        /// </summary>
        public ExperienceStreamState State
        {
            get
            { return this.state; }

            private set
            {
                if (this.state != value)
                {
                    var eventArgs = new ESStateChangedEventArgs(this.state, value);
                    this.state = value;
                    if (StateChanged != null)
                    {
                        StateChanged(this, eventArgs);
                    }
                }
            }
        }

        public event EventHandler<ESStateChangedEventArgs> StateChanged;

        /// <summary>
        /// The UI content to be shown by player on "stage". ES should draw all its content into this control.
        /// This could be set to null if the ES has no UI to display (such as Audio only ES).
        /// </summary>
        public FrameworkElement UserInterfaceControl
        {
            get { return this; }
        }
        #endregion

        private void WriteCurrentTimeStatus()
        {
            this.textBlockDebugInfo.Text = String.Format("Debug info: Current logical offset: {0} secs, Time till player start: {1} secs",
                (int)this.orchestrator.CurrentLogicalTimeOffset.TotalSeconds,
                (int)(DateTime.Now - this.startTime).TotalSeconds);
        }

        private static Storyboard CreateMarqueeStoryboard(FrameworkElement control, Panel panelControl)
        {
            DoubleAnimation doubleAnimation = new DoubleAnimation();
            doubleAnimation.From = -control.ActualWidth;
            doubleAnimation.To = panelControl.ActualWidth;
            doubleAnimation.RepeatBehavior = RepeatBehavior.Forever;
            doubleAnimation.Duration = new Duration(TimeSpan.FromSeconds(5));
            Storyboard storyboard = new Storyboard();
            storyboard.Children.Add(doubleAnimation);
            Storyboard.SetTarget(doubleAnimation, control);
            Storyboard.SetTargetProperty(doubleAnimation, new PropertyPath("(Canvas.Left)"));
            return storyboard;
        }


        private ExperienceStreamData esData;
        private IOrchestrator orchestrator;
        private Storyboard storyboard;
        private DispatcherTimer timer;
        private ExperienceStreamState state;
        private DateTime startTime;
        private bool isStoryboardStarted = false;

    }
}
