﻿#region Namespaces
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel.Composition;
using Microsoft.Research.Rin.AuthoringContracts.ViewModel;
using Microsoft.Research.Rin.AuthoringContracts.Interfaces;
using Microsoft.Research.Rin.DataModel;
using System.Collections.ObjectModel;
using System.Windows.Media.Imaging;
using System.Xml.Linq;
using System.Windows.Input;
using Microsoft.Research.Rin.AuthoringContracts.Model.UndoRedoService; 
#endregion

namespace HelloWorldEditor
{
    /// <summary>
    /// Attributes required by MEF for loading the plugins
    /// </summary>
    [Export(typeof(ExperienceStreamEditor))]
    [ExportMetadata("ContentTypes", new string[] { "HelloWorldES" })]
    [ExportMetadata("ProviderTypes", new string[] { "MicrosoftResearch.Rin.Samples.HelloWorldES" })]
    [ExportMetadata("Priority", 0)]
    public class HelloWorldEditor : ExperienceStreamEditor, IXmlEdit
    {
        #region Members

        private ScreenLayout layout { get; set; }
        private IAssetViewModel assetViewModel;
        private List<IAssetViewModel> assetsinUse = new List<IAssetViewModel>();
        private TimeSpan duration = TimeSpan.Zero;
        private ExperienceStreamData experienceStreamData = null;
        private BitmapImage thumbnailImage = null;
        private TimeSpan startTime;
        
        #endregion

        #region Importing Constructor

        [ImportingConstructor]
        public HelloWorldEditor(IAssetViewModel assetViewModel, ExperienceStreamData experienceStreamData, string segmentId, ICoreUtilities utilities)
            : base(segmentId, "MicrosoftResearch.Rin.Samples.HelloWorldES", utilities)
        {
            this.experienceStreamData = new ExperienceStreamData();
            if (assetViewModel != null)
            {
                this.experienceStreamData.Id = base.GetNextUniqueExperienceStreamId();
                this.assetViewModel = assetViewModel;
                this.assetsinUse.Add(this.assetViewModel);
                this.assetViewModel.PropertyChanged += new System.ComponentModel.PropertyChangedEventHandler(assetViewModel_PropertyChanged);

                this.experienceStreamData.Header.Provider.Name = base.ProviderName;
                this.experienceStreamData.Header.Provider.Version = new Version(0, 0);

                KeyframeSequence keySequence = new KeyframeSequence();
                keySequence.Id = "defaultSequence";
                //Add content_type to the keyframesequence auxiliary data
                AuxiliaryData auxData = new AuxiliaryData();
                XElement xelm = new XElement(assetViewModel.AssetType.ContentType);
                auxData.Add("ContentType", xelm);
                keySequence.Data = auxData;
                this.experienceStreamData.KeyframeSequences.Add(keySequence.Id, keySequence);
                this.experienceStreamData.Header.DefaultKeyframeSequence = keySequence.Id;

                UpdateDuration(false);

                this.IsDirty = true;
            }
            else
            {
                InitWithExperienceStreamData(experienceStreamData);
            }
        }

        #endregion

        #region Initialization

        /// <summary>
        /// Private method for initializing members.
        /// </summary>
        private void InitWithExperienceStreamData(ExperienceStreamData experienceStreamData)
        {
            KeyframeSequence sequence = experienceStreamData.KeyframeSequences["defaultSequence"];

            if (this.assetViewModel != null)
            {
                this.assetViewModel.PropertyChanged -= new System.ComponentModel.PropertyChangedEventHandler(assetViewModel_PropertyChanged);
            }

            this.assetsinUse.Clear();

            this.assetViewModel = base.ResolveProviderToAssetViewModel(this.ProviderName);

            this.experienceStreamData = experienceStreamData;

            UpdateDuration(true);

            InitTranstionData(this.experienceStreamData);   

            this.IsDirty = false;
        }

        #endregion

        #region EventHandlers

        /// <summary>
        /// This happens when the underlying asset is reimported and we need to update the thumbnail.
        /// </summary>
        /// <param name="sender">Asset view model</param>
        /// <param name="e">Property that is changed</param>
        void assetViewModel_PropertyChanged(object sender, System.ComponentModel.PropertyChangedEventArgs e)
        {
            // Handle the model change and update the properties
            if (e.PropertyName.Equals("ThumbnailImage", StringComparison.InvariantCulture))
            {
                this.thumbnailImage = null;
                OnPropertyChanged("Thumbnail");
            }
        }

        #endregion

        # region Private Methods

        /// <summary>
        /// update the duration of this ES and notify the change to the UI
        /// </summary>
        private void UpdateDuration(bool isXmlDurationPriority)
        {
            TimeSpan totaltime = TimeSpan.Zero;
            if (isXmlDurationPriority)
            {
                if (this.experienceStreamData.KeyframeSequences["defaultSequence"].Header.Duration.TotalSeconds > 0.0)
                {
                    totaltime = this.experienceStreamData.KeyframeSequences["defaultSequence"].Header.Duration;
                }
                else if (assetViewModel != null && assetViewModel.FileDuration.TotalSeconds > 0.0)
                {
                    totaltime = assetViewModel.FileDuration;
                }
            }
            else
            {
                if (assetViewModel != null && assetViewModel.FileDuration.TotalSeconds > 0.0)
                {
                    totaltime = assetViewModel.FileDuration;
                }
            }

            //Set the default duration of the editor if there are no keyframes.
            totaltime = totaltime == TimeSpan.Zero ? TimeSpan.FromSeconds(10) : totaltime;
            this.duration = totaltime;
            OnPropertyChanged("Duration");
        }

        /// <summary>
        /// This method allows to set the isdirty on the base class which is only protected
        /// Editor keyframes and its child classes needs to update the editor dirty and cannot access due to protection level
        /// Hence using this method we open the access to these clases.
        /// </summary>
        internal void SetEditorDirty()
        {
            this.IsDirty = true;
        }

        #endregion

        #region ITimelineExperienceStream Members

        /// <summary>
        /// Thumbnail of the photosynth. 
        /// Returns null if no keyframe has been captured.
        /// </summary>
        public override BitmapImage Thumbnail
        {
            get
            {
                if (this.thumbnailImage == null && assetViewModel != null)
                {
                    this.thumbnailImage = assetViewModel.ThumbnailImage;
                }
                return this.thumbnailImage;
            }
        }

        /// <summary>
        /// Title
        /// </summary>
        public override string Title
        {
            get { return this.assetViewModel != null ? this.assetViewModel.Title : string.Empty; }
            set { }
        }

        /// <summary>
        /// Duration of the trajectory
        /// </summary>
        public override TimeSpan Duration
        {
            get
            {
                return this.duration;
            }
            set
            {
                this.duration = value;
                OnPropertyChanged("Duration");
            }
        }

        /// <summary>
        /// Type of asset
        /// </summary>
        public override IAssetType AssetType
        {
            get { return this.assetViewModel != null ? this.assetViewModel.AssetType : null; ; }
        }

        /// <summary>
        /// Collection of keyframes for this trajectory
        /// </summary>
        public override ObservableCollection<ITimelineKeyframe> TimelineKeyframes
        {
            get { return null; }
        }

        /// <summary>
        /// Gets a value indicating if the duration of this experience stream can be changed out side the editor from the sceneline.
        /// </summary>
        public override bool AllowDurationChange
        {
            get
            {
                return true;
            }
            set
            { }
        }

        /// <summary>
        /// Gets or sets the start time.
        /// </summary>
        public override TimeSpan StartTime
        {
            get { return this.startTime; }
            set
            {
                this.startTime = value;
                OnPropertyChanged("StartTime");
            }
        }

        /// <summary>
        /// Gets or sets the layout(Not used currently).
        /// </summary>
        public override ScreenLayout Layout
        {
            get { return this.layout; }
            set { this.layout = value; }
        }

        /// <summary>
        /// Since we do not allow duration change of the entire synth with out editor, thow not implemented.
        /// Read AllowDurationChange
        /// </summary>
        protected override TimeSpan MinDuration
        {
            get
            {
                return new TimeSpan(0, 0, 2);
            }
        }

        /// <summary>
        /// Since we do not allow duration change of the entire synth with out editor, thow not implemented.
        /// Read AllowDurationChange
        /// </summary>
        protected override TimeSpan MaxDuration
        {
            get
            {
                return new TimeSpan(0, 0, 100);
            }
        }

        /// <summary>
        /// Returns the Experience stream data
        /// </summary>
        /// <returns></returns>
        public override ExperienceStreamData GetExperienceStreamData()
        {
            if (this.IsDirty)
            {
                KeyframeSequence keySequence = experienceStreamData.KeyframeSequences["defaultSequence"];
                keySequence.Header.Duration = this.duration;
                keySequence.Keyframes.Clear(); //clear the keyframes.
                SetTransitionData(this.experienceStreamData);

                this.IsDirty = false;
            }
            return experienceStreamData;
        }

        /// <summary>
        /// This returns the lists of assets used in the Experience stream editor.
        /// </summary>
        /// <returns></returns>
        public override IList<IAssetViewModel> GetAssetsUsed()
        {
            return this.assetsinUse;
        }
        /// <summary>
        /// A helper function to add the given  resource to the RequiredResourceOnLoad of the exp stream data we are working with
        /// </summary>
        /// <param name="resToAdd"></param>
        public void AddRequiredResourceOnLoad(ResourceReference resToAdd)
        {
            this.experienceStreamData.RequiredResourcesOnLoad.Add(resToAdd);
            this.IsDirty = true;
        }

        /// <summary>
        /// Adds auxiliary data to the current experience stream
        /// </summary>
        /// <param name="elementName"></param>
        /// <param name="element"></param>
        public void AddExperienceStreamAuxData(string elementName, XElement element)
        {
            this.experienceStreamData.Data.Add(elementName, element);
            this.IsDirty = true;
        }
        
        /// <summary>
        /// Gets the collection of metadata of this experience that the user can modify.
        /// </summary>
        public override Collection<ExperienceMetadata> EditableMetadataCollection
        {
            get { throw new NotImplementedException(); }
        }

        #endregion
        
        #region IXMLEdit
        /// <summary>
        /// Return the Experience stream Data
        /// </summary>
        public string Data
        {
            get
            {
                return XmlEditorHelper<ExperienceStreamData>.GetObjectString(this.GetExperienceStreamData());
            }
        }

        /// <summary>
        /// Sets the experience stream data
        /// </summary>
        /// <param name="data">experience stream Data</param>
        /// <returns>if set was successful or not</returns>
        public bool SetData(string data)
        {
            string error = string.Empty;
            ExperienceStreamData esData = XmlEditorHelper<ExperienceStreamData>.GetObjectData(data, out error);
            if (esData != null)
            {
                IRinAction action = new UndoableSetDataAction<ExperienceStreamData>(this.experienceStreamData, esData,
                    (expData) => { InitWithExperienceStreamData(expData); this.IsDirty = true; });
                base.UndoRedoService.ExecuteCommand(action, null);
                return true;
            }
            else
            {
                return false;
            }
        }

        /// <summary>
        /// Checks if the data is valid Experience stream data
        /// </summary>
        /// <param name="data">Experience stream data XML</param>
        /// <param name="error">Any error during validataion</param>
        /// <returns>true or false</returns>
        public bool IsValid(string data, out string error)
        {
            error = string.Empty;
            ExperienceStreamData esData = XmlEditorHelper<ExperienceStreamData>.GetObjectData(data, out error);
            if (esData == null)
            {
                return false;
            }
            else
            {
                return true;
            }

        }
        #endregion
    }

}
