﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using MicrosoftResearch.Rin.RinPlayerContracts;
using Microsoft.Research.Rin.DataModel;
using GarboChess;
using System.Xml.Linq;
using System.Diagnostics;
using System.ComponentModel.Composition;
using ChessInteractionControls;

namespace ChessES
{
    /// <summary>
    /// Sample to show how to build an Experience Stream that works based on discrete keyframes. System base class DiscreteKeyframeESBase
    /// gives default implementation for all IExperienceStream members and takes care of parsing keyframes element and calling
    /// DisplayKeyframe as right times. Using this base class significantly reduces coding.
    /// 
    /// This sample also show how to use interaction controls and get the controls from orchestrator
    /// </summary>
    public partial class ChessES : DiscreteKeyframeESBase, IAuthorableExperienceStream, IChessActionNotificationSubscriber, IInteractiveExperienceStream
    {
        public ChessES(ExperienceStreamData esData, IOrchestrator orchestrator): base(esData, orchestrator)
        {
            InitializeComponent();
            
            bool isAuthoring = (this.orchestrator.PlayerConfiguration.PlayerMode == PlayerMode.AuthorerEditor);
            stackLastMoveInfo.Visibility = isAuthoring ? Visibility.Visible : Visibility.Collapsed;

            this.chessInteractionControl = new List<IInteractionControl>();
        }

        /// <summary>
        /// Returns current state as a keyframe to be stored in RIN data file. Used in authoring scenario.
        /// </summary>
        public KeyframeData GetCapturedKeyframe(KeyframeData keyframeData)
        {
            XElement dataElement = null;
                dataElement = new XElement("kf",
                    new XAttribute("Fen", p.GetFen()), new XAttribute("MoveInfo", this.txtLastMoveString.Text.Trim()));

            keyframeData.Data["default"] = dataElement;
            return keyframeData;
        }


        /// <summary>
        /// Override for base Load. Sets up chess board and controls.
        /// </summary>
        public override void Load(TimeSpan offset)
        {
            base.Load(offset);

            p = new Position();
            p.ResetBoard();
            cb.Position = p;

            searcher = new Searcher(p);
            searcher.AllottedTime = 200;

            cb.UserMove += new SilverlightChess.ChessBoard.UserMoveHandler(cb_UserMove);
            cb.MouseLeftButtonDown += new MouseButtonEventHandler(cb_MouseLeftButtonDown);
            this.MouseLeftButtonDown += cb_MouseLeftButtonDown;
            UpdateNextMoveInfo();
            this.movesList = new MovesList(listBoxMoves, this.keyframes, this.orchestrator, this);


            //Get the chess interaction control from orchestrator
            IInteractionControl chessControl = this.orchestrator.GetInteractionControl(ChessInteractionControl.ControlName, this);

             
            if (this.chessInteractionControl != null)
            {
                //Clear the controls in case already added (this is in case when load is called again)
                if (this.chessInteractionControl.Count > 0)
                {
                    this.chessInteractionControl.Clear();
                }

                //Add the new control to list of controls
                if (chessControl != null)
                {

                    this.chessInteractionControl.Add(chessControl);
                }
            }
            this.State = ExperienceStreamState.Ready;
        }

        /// <summary>
        /// Pauses narratives for user exploration when a mouse down occurs and informs the player and other experience stream about user interaction event
        /// </summary>
        private void cb_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            this.orchestrator.PauseNarrativeForExplore();
        
            //raise the InteractionActivatedEventId to inform player about interaction activity.
            //raising this event also shows the interaction controls in the player.
            this.orchestrator.OnESEvent(this, ESEventIds.InteractionActivatedEventId, null);
        }

        private void cb_UserMove(ushort move)
        {
            p.MakeMove(move);
            UpdateBoardForNewPosition();
            txtLastMoveString.Text = cb.LastUserMoveNiceString;

            MakeComputerMoveIfNeeded();
        }


        /// <summary>
        /// Updates chessboard when user makes a move.
        /// </summary>
        private void UpdateBoardForNewPosition()
        {
            cb.UpdateForNewPosition();
            UpdateNextMoveInfo();
        }

        /// <summary>
        /// Updates chessboard if computer is playing the next move
        /// </summary>
        private void MakeComputerMoveIfNeeded()
        {
            if (this.orchestrator.PlayerState == PlayerState.PausedForExplore && this.ComputerPlaysNextMove)
            {
                p.MakeMove(searcher.Search());
                UpdateBoardForNewPosition();
            }
        }


        /// <summary>
        /// Updates the message to indiacte who has the next move
        /// </summary>
        private void UpdateNextMoveInfo()
        {
            bool isWhite = p.ToMove == Pieces.White;
            tbNextMove.Text = isWhite ? "Next Move: White" : "Next Move: Black";
        }

        /// <summary>
        /// Override for base Play. Sets the status text on the board
        /// </summary>
        public override void Play(TimeSpan transitionTime)
        {
            base.Play(transitionTime);
            this.cb.SetStatusText();
        }

        /// <summary>
        /// Displays the currently passed keyframe in the chessboard.
        /// </summary>
        protected override void DisplayKeyframe(KeyframeData keyframeData)
        {
            XElement element = keyframeData.Data["default"];
            Debug.WriteLine("Keyframe at chess: " + element.ToString());

            string viewerState = element.Attribute("Fen").Value;
            p.InitializeFromFen(viewerState);
            UpdateBoardForNewPosition();
            this.movesList.LoadKeyframe(keyframeData);
        }


        /// <summary>
        /// Resets the chess board
        /// </summary>
        protected override void ResetUserInterface()
        {
            p.ResetBoard();
            UpdateBoardForNewPosition();
        }

        private Position p;
        private Searcher searcher;
        private MovesList movesList;
        private List<IInteractionControl> chessInteractionControl;


        /// <summary>
        /// Undo the last move 
        /// </summary>
        public void OnUndoLastMove()
        {
            this.cb.UndoLastMove();
            UpdateNextMoveInfo();
        }

        /// <summary>
        /// Indicates if the game is being played against computer
        /// </summary>
        public bool ComputerPlaysNextMove
        {
            get;
            set;
        }

        /// <summary>
        /// Indicates whether to show the chess interaction controls or not
        /// </summary>
        public bool ShowChessControls
        {
            get { return true; }
        }

        /// <summary>
        /// List of interaction controls used by the experience stream
        /// </summary>
        public IEnumerable<IInteractionControl> InteractionControls
        {
            get { return chessInteractionControl; }
        }
    }

    internal class MovesList
    {
        public MovesList(ListBox movesListBox, List<KeyframeData> keyframes, IOrchestrator orchestrator, ChessES chessES)
        {
            this.movesListBox = movesListBox;
            this.keyframes = keyframes;
            this.orchestrator = orchestrator;
            this.chessES = chessES;

            movesListBox.SelectionChanged += new SelectionChangedEventHandler(movesListBox_SelectionChanged);
            LoadMoves();
        }

        public void LoadKeyframe(KeyframeData keyframe)
        {
            var listItem = this.movesListBox.Items.FirstOrDefault(item => ((MoveInfo)item).SeekPointSeconds == keyframe.Header.Offset);
            movesListBox.SelectedItem = listItem;
        }

        void movesListBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            var moveInfo = movesListBox.SelectedItem as MoveInfo;
            if (moveInfo != null)
            {
                this.orchestrator.SeekNarrative(TimeSpan.FromSeconds(moveInfo.SeekPointSeconds), this.chessES);
            }
        }

        private void LoadMoves()
        {
            movesListBox.Items.Clear();
            string lastMoveInfo = String.Empty;
            foreach (var kf in keyframes)
            {
                var moveInfo = kf.Data["default"].GetAttributeValue("MoveInfo", "");
                if (!String.IsNullOrEmpty(moveInfo) && moveInfo != lastMoveInfo)
                {
                    movesListBox.Items.Add(new MoveInfo() { MoveInfoName = moveInfo, SeekPointSeconds = kf.Header.Offset });
                    lastMoveInfo = moveInfo;
                }
            }
        }

        private class MoveInfo
        {
            public string MoveInfoName;
            public double SeekPointSeconds;
            public override string ToString()
            {
                return MoveInfoName;
            }
        }

        private ListBox movesListBox;
        private List<KeyframeData> keyframes;
        private IOrchestrator orchestrator;
        private ChessES chessES;
    }

    [Export(typeof(IExperienceStreamFactory))]
    public class ExperienceStreamFactory : BaseV1ExperienceStreamFactory
    {
        public override IExperienceStream CreateInstance(ExperienceStreamData esData, IOrchestrator orchestrator)
        {
            return new ChessES(esData, orchestrator);
        }

        public override string ProviderTypeId
        {
            get { return "MicrosoftResearch.Rin.Samples.ChessES"; }
        }

    }
}
