﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using MicrosoftResearch.Rin.RinPlayerContracts;
using System.ComponentModel.Composition;

namespace ChessInteractionControls
{
    /// <summary>
    /// Sample to show how to build new interaction controls for any experience stream
    /// </summary>
    public partial class ChessInteractionControl : UserControl, IInteractionControl
    {
        public const string ControlName = "ChessInteractionControl";

        public ChessInteractionControl(IExperienceStream es, IPlayerStyleInfo styleInfo)
        {
            InitializeComponent();

            //Create the control only if the given experience stream is supported for this control
            if (es is IChessActionNotificationSubscriber &&
                styleInfo.ExternalControllerName.Equals(SystemControllerNames.DefaultControllerName, StringComparison.OrdinalIgnoreCase))
            {
                this.experienceStream = es as IChessActionNotificationSubscriber;
            }
            else
            {
                throw new Exception("Experience stream not supported");
            }
        }

        /// <summary>
        /// Indiactes the order in which the control should appear when more than one control is being used
        /// </summary>
        /// <remarks>
        /// Numbers 0 - 1000 are reserved for system interaction controls
        /// </remarks>
        public int AppearanceOrder
        {
            get { return 1001; }
        }

        /// <summary>
        /// UI element which should be added to player UI when user is interacting 
        /// </summary>
        public FrameworkElement Control
        {
            get { return this; }
        }

        /// <summary>
        /// Position of the interaction control
        /// </summary>
        public InteractionControlPlacement Placement
        {
            get { return InteractionControlPlacement.Left; }
        }

        /// <summary>
        /// Load method is called whenever the interaction control is added to player UI
        /// </summary>
        public void Load()
        {
            if (this.experienceStream != null)
            {
                if (experienceStream.ShowChessControls)
                {
                    this.Visibility = Visibility.Visible;
                }
                else
                {
                    this.Visibility = Visibility.Collapsed;
                }
                
                this.ComputerPlaysNextMove.IsChecked = this.experienceStream.ComputerPlaysNextMove;
            }
        }


        /// <summary>
        /// Unload method is called whenever interaction control is removed from the player UI
        /// </summary>
        public void Unload()
        {
            // no op //
        }



        private IChessActionNotificationSubscriber experienceStream;

        /// <summary>
        /// Handler for undo button
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void UndoLastMove_Click(object sender, RoutedEventArgs e)
        {
            if (this.experienceStream != null)
            {
                this.experienceStream.OnUndoLastMove();
            }
        }


        /// <summary>
        /// Handler for play vs computer buttons checked state (this notifies the chess es to play against computer)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ComputerPlaysNextMove_Checked(object sender, RoutedEventArgs e)
        {
            
            if (this.experienceStream != null)
            {
                this.experienceStream.ComputerPlaysNextMove = true;
            }
        }

        /// <summary>
        /// Handler for play vs computer buttons unchecked state (this notifies the chess es to play against human)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ComputerPlaysNextMove_Unchecked(object sender, RoutedEventArgs e)
        {
            if (this.experienceStream != null)
            {
                this.experienceStream.ComputerPlaysNextMove = false;
            }
        }
    }


    /// <summary>
    /// Class for chess interaction control factory (Export this as MEF exportable to register it with the player)
    /// </summary>
    [Export(typeof(IInteractionControlFactory))]
    public class ChessInteractionControlFactory : IInteractionControlFactory
    {
        /// <summary>
        /// Name of the control
        /// </summary>
        public string ControlName
        {
            get { return ChessInteractionControl.ControlName; }
        }

        /// <summary>
        /// Method to create the instance of the control 
        /// </summary>
        /// <param name="experienceStream">Experience stream assosiated with the control</param>
        /// <param name="styleInfo">Style information provided by the player</param>
        /// <returns></returns>
        public IInteractionControl GetInstance(IExperienceStream experienceStream, IPlayerStyleInfo styleInfo)
        {
            IInteractionControl control = null;
            if (experienceStream is IChessActionNotificationSubscriber &&
                styleInfo.ExternalControllerName.Equals(SystemControllerNames.DefaultControllerName))
            {
                control = new ChessInteractionControl(experienceStream, styleInfo);
            }

            return control;
        }
    }



}
